import { IAsyncStore } from '@eppo/js-client-sdk-common';
/**
 * Interface for a string-based storage engine which stores string contents as well as metadata
 * about those contents (e.g., when it was last updated)
 */
export interface IStringStorageEngine {
    getContentsJsonString: () => Promise<string | null>;
    getMetaJsonString: () => Promise<string | null>;
    setContentsJsonString: (configurationJsonString: string) => Promise<void>;
    setMetaJsonString: (metaJsonString: string) => Promise<void>;
}
/**
 * Asynchronous store that operates using a provided underlying engine that stores strings.
 * Example string-based storage engines include a browser's local storage, or a Chrome extension's
 * chrome storage.
 * Objects are stored by serializing to JSON strings, and then retrieved by deserializing.
 */
export declare class StringValuedAsyncStore<T> implements IAsyncStore<T> {
    private storageEngine;
    private cooldownSeconds;
    private initialized;
    constructor(storageEngine: IStringStorageEngine, cooldownSeconds?: number);
    isInitialized(): boolean;
    isExpired(): Promise<boolean>;
    entries(): Promise<Record<string, T>>;
    setEntries(entries: Record<string, T>): Promise<void>;
}
//# sourceMappingURL=string-valued.store.d.ts.map